/*  Дерево автоматизмов

Это дерево активируется при:
1. Всегда при любых событиях с Пульта – так же как дерево рефлексов, но если к ветке привязан автоматизм, то он выполняется преимущественно, блокируя рефлексы потому, что уже было произвольностью преодолено действие рефлекса при выработке автоматизма. Такой автоматизм обладает меткой успешности ==1. Успешность ==0 означает предположительный вариант действий, а успешность < 0 – заблокированный вариант действия. Так что к ветке может быть прикреплено множество неудачных и предположительных автоматизмов и только один удачный. Более удачный результат переводит ранее удачный автоматизм в предполагаемые.
2. При произвольной активации отдельных условий.
Отсуствие подходящей для данных условий ветки дерева вызывает Ориентировочный рефлекс привлечения внимания к активной ветке с осмыслением ситуации
и рассмотрением альтернатив действиям (4 уровня глубины рассмотрения).
При формировании нового предположительного действия создается новая ветка дерева и к ней прикрепляется автоматизм.
Т.е. новые условия не создают новой ветки, а тольно - новый автоматизм, а пока нет автоматизма будет ориентировочный рефлекс.

У дерева фиксированных 6 уровней:
нулевой - основание
Базовое состояние - 3 вида
Эмоция
Активность с Пульта - образ ActivityFromIdArr=make(map[int]*Activity)
Образ контекста сообщения: сочетание Tone и Mood из структуры vrbal
Первый символ фразы
Фраза - VerbalID

Для оптимизации поиска по дереву перед узлом Verbal идет узел первого символа : var symbolsArr из word_tree.go
*/

package psychic

import (
	"BOT/brain/gomeostas"
)

// инициализирующий блок - в порядке последовательности инициализаций
// из psychic.go
func automatizmTreeInit(){

	loadAutomatizmTree()

}
/////////////////////////////////////////////////////////////

////// ДЕРЕВО автоматизмов имеет фиксированных 6 уровней (кроме базового нулевого)
type AutomatizmNode struct { // узел дерева автоматизмов
	ID int
	BaseID int // базовое состояние
	EmotionID int // стиль поведения - сочетание активностей Базовых контекстов  - ID BaseStyleArr
	ActivityID int // образ сочетания действия с Пульта
	ToneMoodID int  // образ контекста сообщения: сочетание Tone и Mood из структуры vrbal
	SimbolID int
	VerbalID int

	Children []AutomatizmNode // дочерние узлы (ветвление) НЕ АДРЕСА, А РЕАЛЬНЫЕ ОБЪЕКТЫ
	ParentID int     // ID родителя
	ParentNode *AutomatizmNode  // адрес родителя
}
var AutomatizmTree AutomatizmNode
var AutomatizmTreeFromID=make(map[int]*AutomatizmNode)
// последовательность узлов активной ветки
var ActiveBranchNodeArr []int
////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////
/* попытка активации дерева автоматизмов, если неудачно - начать искать вариант действий
Используется активная текущая информационная среда из psychic.go:
var PsyBaseID=0 // текущее базовое состояние, может быть произвольно изменено
var PsyEmotionImg *Emotion // текущая эмоция Emotion, может быть произвольно изменена
var PsyActionImg *Activity // текущий образ сочетания действий с Пульта Activity
var PsyVerbImg *Verbal // текущий образ фразы с Пульта Verbal
*/
var detectedActiveLastNodID=0
// нераспознанный остаток - НОВИЗНА
var CurrentAutomatizTreeEnd []int
var currentStepCount=0
func automatizmTreeActivation()(int){
	if PulsCount<4{// не активировать пока все не устаканится
		return 0
	}
	detectedActiveLastNodID=0
	ActiveBranchNodeArr=nil
	CurrentAutomatizTreeEnd=nil
	currentStepCount=0

	var ToneMoodID=0
	if CurrentInformationEnvironment.PsyVerbImg!=nil {
		ToneMoodID = getToneMoodID(CurrentInformationEnvironment.PsyVerbImg.ToneID, CurrentInformationEnvironment.PsyVerbImg.MoodID)
	}

	// вытащить 3 уровня условий в виде ID их образов
	var lev1=gomeostas.CommonBadNormalWell
	var lev2=0
	if CurrentInformationEnvironment.PsyActionImg!=nil{
		lev2=CurrentInformationEnvironment.PsyEmotionImg.ID
	}

	var lev3=0
	if CurrentInformationEnvironment.PsyActionImg!=nil{
		lev3=CurrentInformationEnvironment.PsyActionImg.ID
	}
	var lev4=ToneMoodID

	var lev5=0
	if CurrentInformationEnvironment.PsyVerbImg!=nil{
		lev5=CurrentInformationEnvironment.PsyVerbImg.ID
	}
	/* для дерева берется только первая фраза, остальные можно восстановить для сопоставлений из
	AutomatizmNode.VerbalID.PhraseID[]
	или из памяти о воспринятых фразах (Vernike_detector.go): var MemoryDetectedArr []MemoryDetected
	 */
	var lev6=0
	if CurrentInformationEnvironment.PsyVerbImg!=nil{
		lev6=CurrentInformationEnvironment.PsyVerbImg.PhraseID[0]
	}
	condArr:=getActiveConditionsArr(lev1, lev2, lev3, lev4, lev5, lev6)
	// основа дерева
	cnt := len(AutomatizmTree.Children)
	for n := 0; n < cnt; n++ {
		node := AutomatizmTree.Children[n]
		lev1 := node.BaseID
		if condArr[0] == lev1 {
			detectedActiveLastNodID=node.ID
			ost:=condArr[1:]
			if len(ost)==0{

			}

			conditionAutomatizmFound(1,ost, &node)

			break // другие ветки не смотреть
		}
	}
// Новая ветка создается только если для нее есть автоматизм!

	// результат поиска:
	if detectedActiveLastNodID>0{
// есть ли неучтенные условия?
		conditionsCount:=getConditionsCount(condArr)
		CurrentAutomatizTreeEnd=condArr[currentStepCount:] // НОВИЗНА
		if currentStepCount<conditionsCount { // не пройдено до конца имеющихся условий
			// автоматизма нет у недоделанной ветки
			automatizm := orientation_1()
			if automatizm !=nil  {
				// нарастить недостающее в ветке дерева - всегда для orientation_1()
				detectedActiveLastNodID = formingBranch(detectedActiveLastNodID, currentStepCount, condArr)
				automatizm.BranchID = detectedActiveLastNodID
				// сразу запустить МОТОРНОЕ действие - в  orientation_1()
				return automatizm.ID // блокировка рефлексов
			}
			// если нет неучтенных условий
		}else{// все условия пойдены,. ветка существует,
			//АВТОМАТИЗМ ДОЛЖЕН БЫТЬ ОБЯЗАТЕЛЬНО потому, что новая ветка создается только если для нее есть автоматизм
			automatizmID := getAutomatizmFromNodeID(detectedActiveLastNodID)
			if automatizmID > 0 {//ориентировочный рефлекс 2
				// проверить подходит ли автоматизм к текущим условиям, если нет, - режим нахождения альтернативы  - ориентировочный рефлекс 2
				automatizm := orientation_2(automatizmID)
				if automatizm !=nil {
					//нарастить недостающую ветку
					nodeID:=formingBranch(detectedActiveLastNodID, currentStepCount, condArr)
					automatizm.BranchID=nodeID
					// сразу попробовать выполнить этот автоматизм, т.к. если вернуло автоматизм, значит хочет попробовать
					RumAutomatizmID(automatizm.ID)
				}
				return automatizmID // блокировка рефлексов
			}
		}
	}else{// вообще нет совпадений для данных условий
//нарастить недостающую ветку - только если создается автоматизм
		// автоматизма нет у недоделанной ветки
		CurrentAutomatizTreeEnd=condArr // все - новизна
		automatizm := orientation_1()
		if automatizm !=nil {
			// нарастить недостающее в ветке дерева - всегда для orientation_1()
			detectedActiveLastNodID = formingBranch(detectedActiveLastNodID, currentStepCount, condArr)
			automatizm.BranchID = detectedActiveLastNodID
			// сразу запустить МОТОРНОЕ действие - в  orientation_1()
			return automatizm.ID // блокировка рефлексов
		}

// 		formingBranch(AutomatizmTree.ID, currentStepCount, condArr)
	}

	return 0
}
//////////////////////////////////////////////////////////////////

func conditionAutomatizmFound(level int,cond []int,node *AutomatizmNode){
	if cond==nil || len(cond)==0{
		return
	}

	ost:=cond[1:]

	for n := 0; n < len(node.Children); n++ {
		cld := node.Children[n]
		var levID = 0
		switch level {
		case 0:
			levID = cld.BaseID
		case 1:
			levID = cld.EmotionID
		case 2:
			levID = cld.ActivityID
		case 3:
			levID = cld.SimbolID
		case 4:
			levID = cld.ToneMoodID
		case 5:
			levID = cld.VerbalID
		}
		if cond[0] == levID {
			detectedActiveLastNodID=cld.ID
			ActiveBranchNodeArr=append(ActiveBranchNodeArr,cld.ID)
		}

		level++
		currentStepCount=level
		conditionAutomatizmFound(level,ost, &node.Children[n])
		return // раз совпало, то другие ветки не смотреть
	}

	return
}
////////////////////////////////////////////////////////






